; -----------------------------------------------------------------------------
; ScanKeys
; Escanea las teclas de control y devuelve las pulsadas.
; Salida:   D -> Teclas pulsadas.
;           Bit 0 -> A pulsada 0/1.
;           Bit 1 -> Z pulsada 0/1.
;           Bit 2 -> 0 pulsada 0/1.
;           Bit 3 -> O pulsada 0/1.
; Altera el valor de los registros AF y D.
; -----------------------------------------------------------------------------
ScanKeys:
ld      a, $f7              ; Carga en A la semifila 1-5
in      a, ($fe)            ; Lee el estado de la semifila
bit     $00, a              ; Comprueba si se ha pulsado el 1
jr      nz, scanKeys_2      ; Si no se ha pulsado salta
; Se ha pulsado; cambia la velocidad de la bola 1 (r?pido)
ld      a, (ballSetting)    ; Carga la configuraci?n de la bola en A
and     $cf                 ; Pone los bits de velocidad a 0
or      $10                 ; Pone los bits de velocidad a 1
ld      (ballSetting), a    ; Carga el valor en memoria
jr      scanKeys_speed      ; Salta para comprobar los controles
scanKeys_2:
bit     $01, a              ; Comprueba si se ha pulsado el 2
jr      nz, scanKeys_3      ; Si no se ha pulsado salta
; Se ha pulsado; cambia la velocidad de la bola 2 (medio)
ld      a, (ballSetting)    ; Carga la configuraci?n de la bola en A
and     $cf                 ; Pone los bits de velocidad a 0
or      $20                 ; Pone los bits de velocidad a 2
ld      (ballSetting), a    ; Carga el valor en memoria
jr      scanKeys_speed      ; Salta para comprobar los controles
scanKeys_3:
bit     $02, a              ; Comprueba si se ha pulsado el 3
jr      nz, scanKeys_ctrl   ; Si no se ha pulsado salta
; Se ha pulsado; cambia la velocidad de la bola 3 (lento)
ld      a, (ballSetting)    ; Carga la configuraci?n de la bola en A
or      $30                 ; Pone los bits de velocidad a 3
ld      (ballSetting), a    ; Carga el valor en memoria

scanKeys_speed:
ld      a, $00              ; Pone A = 0
ld      (countLoopBall), a  ; Pone el contador de iteraciones para la bola a 0
scanKeys_ctrl:
ld      d, $00              ; Pone el registro D a 0.

scanKeys_A:
ld      a, $fd          	; Carga en A la semi fila A-G
in      a, ($fe)        	; Lee el estado de la semi fila
bit     $00, a          	; Comprueba si se ha pulsado la A
jr      nz, scanKeys_Z  	; Si no se ha pulsado, salta
set     $00, d          	; Pone a 1 el bit correspondiente a la A

scanKeys_Z:
ld      a, $fe          	; Carga en A la semi fila CS-V
in      a, ($fe)        	; Lee el estado de la semi fila
bit	    $01, a          	; Comprueba si se ha pulsado la Z
jr      nz, scanKeys_0  	; Si no se ha pulsado, salta
set     $01, d         		; Pone a 1 el bit correspondiente a la Z

; Comprueba que no se hayan pulsado las dos teclas de dirección
ld      a, d            	; Carga el valor de D en A
sub     $03             	; Resta 3 a A para comprobar si se han pulsado la A y la Z a la vez
jr      nz, scanKeys_0  	; Si no se han pulsado, salta
ld      d, a            	; Pone D a 0

scanKeys_0:
ld      a, $ef          	; Carga la semi fila 0-6
in      a, ($fe)        	; Lee el estado de la semi fila
bit     $00, a          	; Comprueba si se ha pulsado el 0
jr      nz, scanKeys_O  	; Si no se ha pulsado, salta
set     $02, d          	; Pone a 1 el bit correspondiente al 0

scanKeys_O:
ld      a, $cf          	; Carga la semi fila P-Y
in      a, ($fe)        	; Lee el estado de la semi fila
bit     $01, a          	; Comprueba si se ha pulsado la O
ret     nz              	; Si no se ha pulsado, salta
set     $03, d          	; Pone a 1 el bit correspondiente a la O

; Comprueba que no se hayan pulsado las dos teclas de dirección
ld      a, d            	; Carga el valor de D en A
and     $0c             	; Se queda con los bits correspondientes a 0 y O
cp      $0c             	; Comprueba si se han pulsado las dos teclas
ret     nz              	; Si no se han pulsado, sale
xor		d					; Se queda con las pulsaciones de A y Z
ld      d, a            	; Carga el valor en D


ret

; -----------------------------------------------------------------------------
; WaitStart.
; Espera que se pulse la tecla 5 para empezar la partida.
; Altera el valor del registro AF.
; -----------------------------------------------------------------------------
WaitStart:
ld      a, $f7          ; Carga en A la semifila 1-5
in      a, ($fe)        ; Lee el teclado
bit     $04, a          ; Eval?a si se ha pulsado el 5
jr      nz, WaitStart   ; Bucle hasta que se pulse el 5

ret