; -----------------------------------------------------------------------------
; Evalúa si hay colisión entre la bola y las palas.
; Altera el valor de los registros AF, C y HL.
; -----------------------------------------------------------------------------
CheckBallCross:
ld      a, (ballSetting)            ; Carga la dirección/velocidad de la bola en A
and     $40                         ; Se queda con el bit 6 (izquierda/derecha)
jr      nz, checkBallCross_left     ; Si no está a 0 va hacia la izquierda y salta

checkBallCross_right:
ld      c, CROSS_RIGHT              ; Carga la columna de colisión en C			
call    CheckCrossX                 ; Evalúa si se produce colisión en el eje X	
ret     nz                          ; Si no se produce, fin de la rutina		
ld      hl, (paddle2pos)            ; Carga la posición de la pala 2 en HL		
call    CheckCrossY                 ; Evalúa si se produce colisión en el eje Y	
ret     nz                          ; Si no se produce colisión, fin de la rutina	

; Si llega aquí hay colisión
ld      a, (ballSetting)            ; Carga la dirección/velocidad de la bola en A
or      $40                         ; Cambia la dirección, la pone hacia la izquierda
ld      (ballSetting), a            ; Carga el valor en memoria
ld      a, $ff                      ; Cambia la rotación de la bola
ld      (ballRotation), a           ; La carga en memoria
ret                                 ; Fin de la rutina

checkBallCross_left:
; La bola va hacia la izquierda
ld      c, CROSS_LEFT               ; Carga la columna de colisión en C			
call    CheckCrossX                 ; Evalúa si se produce colisión en el eje X
ret     nz                          ; Si no se produce, fin de la rutina
ld      hl, (paddle1pos)            ; Carga la posición de la pala 1 en HL
call    CheckCrossY                 ; Evalúa si se produce colisión en el eje Y
ret     nz                          ; Si no se produce colisión, fin de la rutina	

; Si llega aquí hay colisión
ld      a, (ballSetting)            ; Carga la dirección/velocidad de la bola en A
and     $bf                         ; Cambia la dirección, la pone hacia la derecha
ld      (ballSetting), a            ; Carga el valor en memoria
ld      a, $01                      ; Cambia la rotación de la bola
ld      (ballRotation), a           ; La carga en memoria
ret                                 ; Fin de la rutina

; -----------------------------------------------------------------------------
; Evalúa si la bola colisiona en el eje X con la pala.
; Entrada:  C -> Columna dónde se produce la colisión. 
; Salida:   Z -> Colisiona.
;           NZ -> No colisiona.
; Altera el valor de los registros AF.
; -----------------------------------------------------------------------------
CheckCrossX:
ld      a, (ballPos)                ; Carga la línea y columna donde está la bola
and     $1f                         ; Se queda con la columna
cp      c                           ; Lo compara con la columna de colisión

ret

; -----------------------------------------------------------------------------
; Evalúa si la bola colisiona en el eje Y con la pala.
; Entrada:  HL -> Posición de la pala	
; Salida:   Z -> Colisiona.
;           NZ -> No colisiona.
; Altera el valor de los registros AF, BC y HL.
; -----------------------------------------------------------------------------
CheckCrossY:
call    GetPtrY                     ; Obtiene la posición vertical de la pala (TTLLLSSS)
; La posición devuelta apunta al primer scanline de la pala que está a 0,
; apunta al siguiente
inc     a
ld      c, a                        ; Carga el valor en C
ld      hl, (ballPos)               ; Carga en HL la posición de la bola
call    GetPtrY                     ; Obtiene la posición vertical de la bola (TTLLLSSS)
ld      b, a                        ; Carga el valor en B
; Comprueba si la bola pasa por encima de la pala
; La bola está compuesta de 1 scanline a 0, 4 a $3c y otro a 0
; La posición apunta al 1er scanline, y se comprueba la colisión con el 5º
add     a, $04                      ; Apunta la posición de la bola al 5º scanline
sub     c                           ; Resta a la posición de la bola, la posición de la pala
ret     c                           ; Si hay acarreo sale porque la bola pasa por encima
; Comprueba si la bola pasa por debajo de la pala
ld      a, c                        ; Carga la posición vertical de la pala en A
add     a, $16                      ; Le suma 22 para apuntar al penúltimo scanline,
                                    ; último que no es 0
ld      c, a                        ; Lo vuelve a cargar en C
ld      a, b                        ; Carga la posición vertical de la bola
inc     a                           ; Le suma 1 para apuntar el scanline 1, primero que no es 0
sub     c                           ; Resta a la posición de la bola, la posición de la pala
ret     nc                          ; Si no hay acarreo la bola pasa por debajo de la pala
                                    ; o colisiona en el último scanline.
                                    ; En este último caso se activa el flag Z
; Hay colisión
xor     a                           ; Activa el flag Z
ret

; -----------------------------------------------------------------------------
; Calcula la posición, rotación y dirección de la bola para pintarla.
; Altera el valor de los registros AF y HL.
; -----------------------------------------------------------------------------
MoveBall:
ld      a, (ballSetting)        ; Carga en A la dirección y velocidad de la bola
and     $80                     ; Comprueba la dirección vertical
jr      nz, moveBall_down       ; Si el bit 7 está a uno, va hacia abajo

moveBall_up:
; La bola va hacia arriba
ld      hl, (ballPos)           ; Carga la posición de la bola en HL
ld      a, BALL_TOP             ; Carga en A el margen superior
call    CheckTop                ; Evalúa si se ha alcanzado el margen superior
jr      z, moveBall_upChg       ; Si se ha alcanzado salta
call    PreviousScan            ; Obtiene el scanline anterior a la posición de la bola
ld      (ballPos), hl           ; Carga en memoria la nueva posición de la bola
jr      moveBall_x              ; Salta

moveBall_upChg:
; La bola va hacia arriba, pero ha llegado al tope y cambia de dirección
ld      a, (ballSetting)        ; Carga en A la dirección y velocidad de la bola
or      $80                     ; Pone la dirección vertical hacia abajo
ld      (ballSetting), a        ; Carga en memoria la nueva dirección de la bola
call    NextScan                ; Obtiene el scanline siguiente a la posición de la bola
ld      (ballPos), hl           ; Carga en memoria la nueva posición de la bola
jr      moveBall_x              ; Salta

moveBall_down:
; La bola va hacia abajo
ld      hl, (ballPos)           ; Carga la posición de la bola en HL
ld      a, BALL_BOTTOM          ; Carga en A el margen superior
call    CheckBottom             ; Evalúa si se ha alcanzado el margen superior
jr      z, moveBall_downChg     ; Si se ha alcanzado salta
call    NextScan                ; Obtiene el scanline siguiente a la posición de la bola
ld      (ballPos), hl           ; Carga en memoria la nueva posición de la bola
jr      moveBall_x              ; Salta

moveBall_downChg:
; La bola va hacia abajo, pero ha llegado al tope y cambia de dirección
ld      a, (ballSetting)        ; Carga en A la dirección y velocidad de la bola
and     $7f                     ; Pone la dirección vertical hacia arriba
ld      (ballSetting), a        ; Carga en memoria la nueva dirección de la bola
call    PreviousScan            ; Obtiene el scanline anterior a la posición de la bola
ld      (ballPos), hl           ; Carga en memoria la nueva posición de la bola

moveBall_x:
ld      a, (ballSetting)        ; Carga en A la dirección y velocidad de la bola
and     $40                     ; Comprueba la dirección horizontal
jr      nz, moveBall_left       ; Si el bit 6 está a uno, va hacia la izquierda

moveBall_right:
; La bola va hacia la derecha
ld      a, (ballRotation)       ; Carga la rotación actual de la bola
cp      $08                     ; Comprueba si ya está en la última rotación
jr      z, moveBall_rightLast   ; Si está en la última rotación salta
inc     a                       ; Incrementa la rotación 
ld      (ballRotation), a       ; La carga en memoria
jr      moveBall_end            ; Fin de la rutina

moveBall_rightLast:
; Está en la última rotación
; Si no ha llegado al límite derecho pone la rotación a 1 
; y pone la bola en la siguiente columna
ld      a, (ballPos)            ; Carga la línea y columna de la bola en A
and     $1f                     ; Se queda solo con la columna
cp      MARGIN_RIGHT            ; Lo comprara con el límite derecho
jr      z, moveBall_rightChg    ; Si lo ha alcanzado salta
ld      hl, ballPos             ; Carga la dirección de la posición de la bola en HL
inc     (hl)                    ; Incrementa la columna
ld      a, $01                  ; Pone la rotación a 1
ld      (ballRotation), a       ; Carga el valor en memoria
jr      moveBall_end            ; Fin de la rutina

moveBall_rightChg:
; Ha llegado al límite derecho, ¡PUNTO!
ld      hl, p1points    	; Carga en HL la dirección de la puntuación del jugador 1
inc     (hl)            	; Lo incrementa
call    PrintPoints     	; Pinta el marcador
call    ClearBall       	; Borra la bola
call    SetBallLeft     	; Pone la bola a la izquierda
jr      moveBall_end    	; Fin de la rutina

moveBall_left:
; La bola va hacia la izquierda
ld      a, (ballRotation)       ; Carga la rotación actual de la bola
cp      $f8                     ; Comprueba si ya está en la última rotación
jr      z, moveBall_leftLast    ; Si está en la última rotación salta
dec     a                       ; Decrementa la rotación 
ld      (ballRotation), a       ; La carga en memoria
jr      moveBall_end            ; Fin de la rutina

moveBall_leftLast:
; Esta en la última rotación
; Si no ha llegado al límite izquierdo pone la rotación a -1 
; y pone la bola en la columna anterior
ld      a, (ballPos)            ; Carga la línea y columna en A
and     $1f                     ; Se queda solo con la columna
cp      MARGIN_LEFT             ; Lo comprara con el límite izquierdo
jr      z, moveBall_leftChg     ; Si lo ha alcanzado salta
ld      hl, ballPos             ; Carga la dirección de la posición de la bola en HL
dec     (hl)                    ; Pasa a la columna anterior
ld      a, $ff                  ; Pone la rotación a -1
ld      (ballRotation), a       ; Carga el valor en memoria
jr      moveBall_end            ; Fin de la rutina

moveBall_leftChg:
; Ha llegado al límite izquierdo, ¡PUNTO!
ld      hl, p2points    	; Carga en HL la dirección de la puntuación del jugador 2
inc     (hl)            	; Lo incrementa
call    PrintPoints     	; Pinta el marcador
call    ClearBall       	; Borra la bola
call    SetBallRight    	; Pone la bola a la derecha

moveBall_end:
ret

; -----------------------------------------------------------------------------
; Calcula la posición de las palas para moverlas.
; Entrada:  D -> Pulsaciones de los controles
; Altera el valor de los registros AF y HL.
; -----------------------------------------------------------------------------
MovePaddle:
bit     $00, d              ; Evalúa si se ha pulsado la A
jr      z, movePaddle_1Down ; Si no se ha pulsado salta
ld      hl, (paddle1pos)    ; Carga en HL la posición de la pala 1
ld      a, PADDLE_TOP       ; Carga en A el margen superior
call    CheckTop            ; Evalúa si se ha alcanzado el margen superior
jr      z, movePaddle_2Up   ; Si se ha alcanzado, salta
call    PreviousScan        ; Obtiene el scanline anterior a la posición de la pala 1
ld      (paddle1pos), hl    ; Carga en memoria la nueva posición de la pala 1
jr      movePaddle_2Up      ; Salta

movePaddle_1Down:
bit     $01, d              ; Evalúa si se ha pulsado la Z               
jr      z, movePaddle_2Up   ; Si no se ha pulsado salta
ld      hl, (paddle1pos)    ; Carga en HL la posición de la pala 1
ld      a, PADDLE_BOTTOM    ; Carga en A el margen inferior
call    CheckBottom         ; Evalúa si se ha alcanzado el margen inferior
jr      z, movePaddle_2Up   ; Si se ha alcanzado, salta
call    NextScan            ; Obtiene el scanline siguiente a la posición de la pala 1
ld      (paddle1pos), hl    ; Carga en memoria la nueva posición de la pala 1

movePaddle_2Up:
bit     $02, d              ; Evalúa si se ha pulsado el 0
jr      z, movePaddle_2Down ; Si no se ha pulsado salta
ld      hl, (paddle2pos)    ; Carga en HL la posición de la pala 2
ld      a, PADDLE_TOP       ; Carga en A el margen superior
call    CheckTop            ; Evalúa si se ha alcanzado el margen superior
jr      z, movePaddle_End   ; Si se ha alcanzado, salta
call    PreviousScan        ; Obtiene el scanline anterior a la posición de la pala 2
ld      (paddle2pos), hl    ; Carga en memoria la nueva posición de la pala 2
jr      movePaddle_End      ; Salta

movePaddle_2Down:
bit     $03, d              ; Evalúa si se ha pulsado la O
jr      z, movePaddle_End   ; Si no se ha pulsado salta
ld      hl, (paddle2pos)    ; Carga en HL la posición de la pala 2
ld      a, PADDLE_BOTTOM    ; Carga en A el margen inferior
call    CheckBottom         ; Evalúa si se ha alcanzado el margen inferior
jr      z, movePaddle_End   ; Si se ha alcanzado, salta
call    NextScan            ; Obtiene el scanline siguiente a la posición de la pala 2
ld      (paddle2pos), hl    ; Carga en memoria la nueva posición de la pala 2

movePaddle_End:
ret

; -----------------------------------------------------------------------------
; Posiciona la bola a la izquierda.
; Altera el valor de los registros AF y HL.
; -----------------------------------------------------------------------------
SetBallLeft:
ld      hl, $4d60           ; Carga en HL la posición de la bola
ld      (ballPos), hl       ; Carga el valor en memoria
ld      a, $01              ; Carga 1 en A
ld      (ballRotation), a   ; Lo carga en memoria, Rotación = 1
ld      a, (ballSetting)    ; Carga en A la dirección y velocidad de la bola
and     $bf                 ; Pone la dirección horizontal hacia la derecha
ld      (ballSetting), a    ; Carga la nueva dirección de la bola en memoria

ret

; -----------------------------------------------------------------------------
; Posiciona la bola a la derecha.
; Altera el valor de los registros AF y HL.
; -----------------------------------------------------------------------------
SetBallRight:
ld      hl, $4d7f           ; Carga en HL la posición de la bola
ld      (ballPos), hl       ; Carga el valor en memoria
ld      a, $ff              ; Carga -1 en A
ld      (ballRotation), a   ; Lo carga en memoria, Rotación = -1
ld      a, (ballSetting)    ; Carga en A la dirección y velocidad de la bola
or      $40                 ; Pone la dirección horizontal hacia la izquierda
ld      (ballSetting), a    ; Carga la nueva dirección de la bola en memoria

ret